
//
// UsageMonitor constructor
//
// updateInterval: Number of seconds between updates
// updateFunction: Callback to receive info
//
function UsageMonitor(updateInterval, updateFunction)
{
    this.updateInterval = updateInterval;
    this.updateFunction = updateFunction;
    this.onerror = null;
    this.onload = null;
}

//
// Method: start()
// Public interface to start your command monitor
//
UsageMonitor.prototype.start = function ()
{
    this.update();
    var usageMonitor = this;
    
    if (this.updateTimer == null) {
        this.updateTimer = setInterval(function() {
            usageMonitor.update();
        }, this.updateInterval * 1000);
    }
}

//
// Method: stop()
// Public interface to stop your monitor
//
UsageMonitor.prototype.stop = function ()
{
    if ( this.updateTimer != null ) {
        clearInterval(this.updateTimer);
        this.updateTimer = null;
    }
}

//
// Method: update()
// Load usage data
//
UsageMonitor.prototype.update = function()
{
    var username = widget.preferenceForKey(dashcode.createInstancePreferenceKey("username"));
    var password = widget.preferenceForKey(dashcode.createInstancePreferenceKey("password"));
    
    if ( username == null || password == null ) {
        return;
    }
    
    if ( this.onload ) {
        this.onload();
    }
    
    var usageMonitor = this;
    
    var client = new XMLHttpRequest();
    client.onreadystatechange = function() {
    
        if ( client.readyState == 4 /* DONE */ && client.status == 200 ) {
            
            if ( client.responseText.indexOf("Login failed") != -1 ) {
                if ( usageMonitor.onerror ) {
                    usageMonitor.onerror("Invalid username or password");
                }
                return;
            }
            
            // Logged in successfully. Open the usage page.
            client.onreadystatechange = function () {
                if ( client.readyState == 4 /* DONE */ && client.status == 200 ) {
         
                    // Got usage page; parse the information
                    var peakMatches = client.responseText.match(/My current plan is:\s*(.*?)\s*<\/td>[\s\S]*?My usage allowance in Peak:\s*([0-9a-zA-Z. ]+)[\s\S]*?Usage:[^0-9]*([0-9a-zA-Z. ]+)[\s\S]*?Full Days remaining in Period:\s*([0-9]+)/);
                    if ( !peakMatches ) {
                        alert("Peak report doesn't match regex.\n\n" + client.getAllResponseHeaders() + "\n" + client.responseText);
                        if ( usageMonitor.onerror ) {
                            usageMonitor.onerror("Unexpected server reply");
                        }
                        return;
                    }
                    
                    // Load off-peak usage as well
                    client.onreadystatechange = function () {
                        if ( client.readyState == 4 /* DONE */ && client.status == 200 ) {
                            // Got off-peak usage page; parse the information
                            var offPeakMatches = client.responseText.match(/My usage allowance in Off-Peak:\s*([0-9a-zA-Z. ]+)[\s\S]*Usage:[^0-9]*([0-9a-zA-Z. ]+)/);
                            if ( !offPeakMatches ) {
                                alert("Off-peak report doesn't match regex.\n\n" + client.responseText);
                                if ( usageMonitor.onerror ) {
                                    usageMonitor.onerror("Unexpected server reply");
                                }
                                return;
                            }
                            
                            // Now have info in peakMatches and offPeakMatches; process it and give to result handler
                            try {
                                var planText = peakMatches[1];
                                var peakQuota = quotaToNumber(peakMatches[2]);
                                var peakUsed = quotaToNumber(peakMatches[3]);
                                var daysRemaining = peakMatches[4];
                                
                                var offpeakQuota = quotaToNumber(offPeakMatches[1]);
                                var offpeakUsed = quotaToNumber(offPeakMatches[2]);
                            } catch ( e ) {
                                alert(e);
                                if ( usageMonitor.onerror ) {
                                    usageMonitor.onerror("Unexpected server reply");
                                }
                                return;
                            }
                            
                            usageMonitor.updateFunction(planText, peakQuota, offpeakQuota, peakUsed, offpeakUsed, daysRemaining);
                            
                        } else if ( client.readyState == 4 /* DONE */ && client.status != 200 ) {
                            if ( usageMonitor.onerror ) {
                                usageMonitor.onerror("Communication problem");
                            }
                        }
                    };
                    client.open("GET", "https://www.unwired.com.au/myaccount/usage.php?view=OFF_PEAK");
                    client.send();
            
                } else if ( client.readyState == 4 /* DONE */ && client.status != 200 ) {
                    if ( usageMonitor.onerror ) {
                        usageMonitor.onerror("Communication problem");
                    }
                }
            };
            
            client.open("GET", "https://www.unwired.com.au/myaccount/usage.php?view=PEAK");
            client.send();
            
        } else if ( client.readyState == 4 /* DONE */ && client.status != 200 ) {
            if ( usageMonitor.onerror ) {
                usageMonitor.onerror("Communication problem");
            }
        }
    };
    
    
    // Log in
    client.open("GET", "https://www.unwired.com.au/myaccount/login.php?ev=nx&redirectPage=myAccount"+
                       "&login_name="+username+"&login_password="+password);
    client.send();
}


function quotaToNumber(quota)
{
    var number = parseFloat(quota);
    if ( !number ) {
        throw "Couldn't parse number from text "+quota;
    }
    
    var type = quota.substr(quota.indexOf(" ")+1, 2).toLowerCase();
    
    if ( type == "kb" ) {
        number *= 1024;
    } else if ( type == "mb" ) {
        number *= 1048576;
    } else if ( type == "gb" ) {
        number *= 1073741824;
    } else {
        alert("quotaToNumber: Type parsing problem: " + type + " (" + quota + ")");
        throw "Couldn't parse type from text "+quota;
    }
    
    return number;
}
